On the Expressive Power of Deep Polynomial Neural Networks
==============================================


This repository contains code in [SageMath](http://www.sagemath.org/) for computing the *dimension of functional varieties* of polynomial networks. 

The code can also be run online [here](https://sagecell.sagemath.org/?z=eJzNWM1v2zYUvxfI_0A4h0qeItTpdgngQ9Auww4JhiHYDoIqUxIdM5ElgZIt20X_973HD4milXToqT7YMvn4Pn7vk5rNZhfvPlXbetcy0m4YqUX1zLKW7xnJ-ZaVDa9KUq3l3npXwk5V0oLsqeCsPeIOJSVru0q8hBfvZsju4t3dHVmSP-68xQf4fPQJuQS-wI2seclb_GFFjoQX77KCNg15UBy8KkXh_g3uEPjkbE2SBA8lidewYh2Qhp9YExB2qKuSle3yGqmJ_oD8R1C04E1LVitJulqRrCpbystGGlHutikTqHjJdqKCVV7ixsBEsKbWEBT0yERj7Dd2EnJXCdCAbuuCBYSrXRQ6MOloQ6LrgHwMyCLG_ZLwlnTVrshJygC0diMYu5ICDOOAdLzdjJVZcwG2KDJtBy-fyLXRPpCyGwZ7uSb72O8NbCjsImG74cLQLTQd2PNpJwSAWRwDknLasIZQwQjizmkBKMLhigzcTkxUkmXH-NOmPacWsFlti2MoA8IcQweGgH-iYV2SgpWe9JLvUMlFIJC_zp7WcEki1CPZ0lbwg3d3FxBQf-GTNTjniF6Vh6PFTRxjBEZKqf9F70g0Zi5fYXJQTA6B4nPitbIqurlaxMHA13cZmygGzubRocjZE8QJIjGi_0I8F80rcu3bebNmLAetOipynTrUSZW_WbsTMvhJtWuhBDiBjpG9WlHIII5ZAgRjf6LNKQRtb_PgHWO5hZ5csjWQcQmWKSyTuuqY8Doyh8VfkO_IYt_OT6n2iDew1gdtJWDVRiSl2QvUt1rDAe6qd0WBq8keEr4SyweQNQ0SJOwOsh3w8EqaFjQBBdVD5wNAgtVQNkBPTM5RAj8JmnN0GaKF4M4V1nNH9rwvr5CPq5UWAqwh4AZuZqPDDaEL1FV61IUEa5AsVxLTDAtlw7e8oGJgAZkMpXFASYkwaxrQ1SrUtdwcg1hwNMaoQMAcl7pUSwI0jZ0xQ4ZHMkoWvi1IWz6V32lYiqprPF8-ZlUBjyr1Upm_g1Gxy6-b4tcN_Loxv67npwGxGF7aqWWVWGwZFB0Iog5T67KAHGQxkh0KPNEAQtANikKGhkUa6CKdHtWDVXwlmze4nIhC_g0eb2TuOGndfD2BbJmig7GQqw5NE9K6ZmXunXwn222IRml_ej3dHQwN72HJt12DkYd-ITXMFcNGzooWa40TnGdRJwvJUpGfhZC9OcIA4vg6Dh_H-BYI7gH6xRPzrgO3-U0BewI-RTxeb2oHqkROUt8DzNjLCgZDXNvxhsHBKt9lrTcunAU4cAHKg0HyEPCtHWY9NsUENiN8ilfxKaA_LUYY6TJ-Xk7VYIgFexQi91NDHxZoCsYZQ-U0d485IYdZPNifGppXmYE0rEz3JusDfFS1QJHcwm5W1UfvPqwpzkaeH2L5gAKhCNDFwnJxKWyX4m5m72auw28jEZAM8brXT1-cGUADdDuNh46D11D5zIQEH4bYanzwZ4Whn23mFiJev3ol28Q0LlNRDsA8WHj8rkiukIZoGkTmXja_h58VFIPEnDyop1dDoyqrLYzfCfYEbx-QFztFzAwjGwbe2aBVmBPmAkTAELjUyfFhj1969HyxsIFrnxnb99pW46BGSsZNVOBfWV5Y_mfZsicm_lENyYMrTrRQkxBK3cf-2KcResaL9hG306HBv3iI4yENGqrix6pb94S4P1YoVhBdXpJHBupdXg6QZeram_T3XE_PvUnH83YDndD874vsAKl9Z37rotzUNGOja7K65FGR8lZQcTSjd8_EHrxKbJTmcvxd7fQZ6DEQlnQr70-7rRdtIXxKCdQWTpmeX5ahfUnp_8p7im-mZn37gF31rNc5yKiK4zDjpVxFk9dTQl_peX6IMX8Dd8Fo3MibNOr7m2xD8vh8oJZzPFIegOT8BmZxDXpmmvcQ5aaNyiPRWb4oqSqe9jI7w8fYZZLUvNwjJ7MQ1g3b5RWsQk9npkA806yCaarUSmJrdyZP2zQ0eYxnYDnRKinPdtGwGNjlw1w3mmQA9Uy43poQg5-UNrxBTb6jd9A7cHQqeg7IB6xdi3HJ47b2xklO4TPaJ-b-okKvv7gdopuAcJDcS_OjRTzNo0cg4gG5iR3_R-pCBkuo1rncvkbBSihjxI9j-SZrBw2krPR7kkG04_Po-dytN94zDlv-lMOVguM4m5_b0qefLjgyHV48R7jU82oif-g2lWYCAzjq_VjS-pOpWVKYNZK02kE71XXH20r66eozKmamBo0XVSVClcbrvUwYgWAymOkC2Vf-m5lUejxNObSKFfEKtpZXKIH9ytcnrbdO-thf4CuFlSlPHqPZRvYQNZ9AiVeV3DCx66XmcqvgH_yHpD_mhIC8P7x38_H9EpYsH48tp6OXuih6tuWl9_Vb8PWbv5yF4CIIH886H9heBZFI_ur2SNhnuyXyLX1iKK8Xbjd91ZDHLsYbrnp5qr5_ldddvee8pIryRRCGYZBv4oGN9WLtGo_2_-HK9wN9_z_8LR6M&lang=sage&interacts=eJyLjgUAARUAuQ==) (scrolling to the bottom to change parameters).


The backpropagation and network class were based on code samples from Michael Nielsen's book ["Neural Networks and Deep Learning"](http://neuralnetworksanddeeplearning.com/).